from base import BaseAssertion


class BeAssertion(BaseAssertion):
  """Determine if two objects are identical."""

  def __init__(self, subject, other):
    self.subject = subject
    self.other = other
    self.passes = subject is other

  @property
  def message(self):
    if self.passes:
      msg = '{subject} is {other}'
    else:
      msg = '{subject} is not {other}'
    return msg.format(
      subject=repr(self.subject),
      other=repr(self.other))



from assertions import BeAssertion as be, BeValidAssertion as be_valid, \
  EqualAssertion as equal, HaveTextAssertion as have_text, \
  RespondToAssertion as respond_to \

from base import RedRoverLiveTest, RedRoverTest
from runner import RedRoverRunner
from subject import describe


__all__ = [
  'RedRoverRunner', 'RedRoverLiveTest', 'RedRoverTest', 'describe', 'be',
  'be_valid', 'equal', 'have_text', 'respond_to']



from be_assertion import BeAssertion
from be_valid_assertion import BeValidAssertion
from equal_assertion import EqualAssertion
from have_text_assertion import HaveTextAssertion
from respond_to_assertion import RespondToAssertion


__all__ = [
  'BeAssertion', 'BeValidAssertion', 'EqualAssertion', 'HaveTextAssertion',
  'RespondToAssertion']



from types import FunctionType

from django.test import LiveServerTestCase, TestCase
import splinter

from subject import _splinter_action


def do_decorate(attr, value):
  protected = [
    'setUp', 'setUpClass', 'tearDown', 'tearDownClass',
    'run', 'skipTest', 'debug']
  return (
    '__' not in attr and
    attr not in protected and
    isinstance(value, FunctionType))


def decorate_all():

  class DecorateAll(type):

    def __new__(cls, name, bases, dct):
      subject = dct.get('subject')
      if subject:
        if subject in dct:
          raise NameError(
            'You cannot use "%s" as a subject name.' % subject)

        if subject == 'page':
          dct['page'] = splinter.Browser('zope.testbrowser')

      for attr, value in dct.iteritems():
        if do_decorate(attr, value):
          if isinstance(value, FunctionType):
            value.__name__ = 'test_%s' % value.__name__

      dct['_browser'] = splinter.Browser('zope.testbrowser')

      return super(DecorateAll, cls).__new__(
        cls, name, bases, dct)

  return DecorateAll


class RedRoverTest(TestCase):

  __metaclass__ = decorate_all()


class RedRoverLiveTest(LiveServerTestCase):

  def __init__(self, *args, **kwargs):
    self.setUp.__globals__.update({'visit': _splinter_action(self, 'visit')})
    super(RedRoverLiveTest, self).__init__(*args, **kwargs)

  __metaclass__ = decorate_all()



from django.core.exceptions import ValidationError
from django.db import models

from base import BaseAssertion


class BeValidAssertion(BaseAssertion):
  """Determine if a Django Model is valid."""

  def __init__(self, subject):
    if not isinstance(subject, models.Model):
      raise ValueError('`be_valid` must be called on a Django Model instance.')

    self.subject = subject
    self.passes = True
    try:
      subject.full_clean()
    except ValidationError:
      self.passes = False

  @property
  def message(self):
    if self.passes:
      msg = '{subject} is valid'
    else:
      msg = '{subject} is not valid'

    return msg.format(subject=repr(self.subject))



from base import BaseAssertion


class HaveTextAssertion(BaseAssertion):
  """Determine if given text appears in an splinter Browser document."""

  def __init__(self, browser, text):
    if not hasattr(browser, 'html'):
      raise ValueError(
        '`have_text` must be called on a page/Browser instance.')
    self.browser = browser
    self.text = text
    self.passes = text in browser.html

  @property
  def message(self):
    if self.passes:
      msg = '{text} appears in {url}'
    else:
      msg = '{text} does not appear in {url}'

    return msg.format(
      text=repr(self.text),
      url=self.browser.url)



import functools


def _splinter_action(_parent, _action):

  _browser = _parent.page

  def do_action(*args, **kwargs):
    if _action == 'visit':
      base_url = _parent.live_server_url
      url = args[0]
      return _browser.visit('%s%s' % (base_url, url))

    return getattr(_browser, _action)(*args, **kwargs)

  return do_action


def _subject(parent=None):

  class Subject(object):

    def __init__(self, value):
      if parent:
        self.value = getattr(parent, value)
      else:
        self.value = value

    def should(self, cls, *args, **kwargs):
      assertion = cls(self.value, *args, **kwargs)
      if not assertion.passes:
        raise AssertionError(assertion.message)
      return True

    def should_not(self, cls, *args, **kwargs):
      assertion = cls(self.value, *args, **kwargs)
      if assertion.passes:
        raise AssertionError(assertion.message)
      return True

  return Subject


def describe(func):

  @functools.wraps(func)
  def test_describe(instance, *args, **kwargs):
    subject_name = getattr(instance, 'subject', None)
    extra_context = {}

    if subject_name:
      subject = getattr(instance, subject_name, None)
      if not subject:
        raise NameError(
          'You are trying to use a subject that has not been initialized ' +
          'in `setUp`.')

      if subject_name == 'page':
        extra_context.update({
          'visit': _splinter_action(instance, 'visit')})

      extra_context.update({
        'it': _subject()(subject),
        'its': _subject(subject)})

      func.__globals__.update(extra_context)

    return func(instance, *args, **kwargs)

  return test_describe



from django.db import models

from base import BaseAssertion


class RespondToAssertion(BaseAssertion):
  """Determine if a Django Model has a particular attribute or method."""

  def __init__(self, subject, attr):
    if not isinstance(subject, models.Model):
      raise ValueError(
        '`respond_to` must be called on a Django Model instance.')

    self.subject = subject
    self.attribute = attr
    fields = [f.__dict__['name'] for f in subject._meta.fields]
    attributes = dir(subject)
    self.passes = attr in fields + attributes

  @property
  def message(self):
    if self.passes:
      msg = '{subject} responds to {attribute}'
    else:
      msg = '{subject} does not respond to {attribute}'

    return msg.format(
      subject=repr(self.subject),
      attribute=repr(self.attribute))
